<?php

/**
 * Custom implementation of comment_Render from comment.module.
 */
function morecomments_comment_render($nid, $page) {
  $GLOBALS['devel_shutdown'] = FALSE;
  global $user;

  $node = node_load($nid);
  $page = intval($page);

  if (empty($node)) {
    return;
  }
  $output = '';

  // Pre-process variables.
  $nid = $node->nid;
  if (empty($nid)) {
    $nid = 0;
  }

  $mode = _comment_get_display_setting('mode', $node);
  $order = _comment_get_display_setting('sort', $node);
  $comments_per_page = _comment_get_display_setting('comments_per_page', $node);

  // Multiple comment view
  $query_count = 'SELECT COUNT(*) FROM {comments} c WHERE c.nid = %d';
  $query = 'SELECT c.cid as cid, c.pid, c.nid, c.subject, c.comment, c.format, c.timestamp, c.name, c.mail, c.homepage, u.uid, u.name AS registered_name, u.signature, u.signature_format, u.picture, u.data, c.thread, c.status FROM {comments} c INNER JOIN {users} u ON c.uid = u.uid WHERE c.nid = %d';

  $query_args = array($nid);
  if (!user_access('administer comments')) {
    $query .= ' AND c.status = %d';
    $query_count .= ' AND c.status = %d';
    $query_args[] = COMMENT_PUBLISHED;
  }

  if ($order == COMMENT_ORDER_NEWEST_FIRST) {
    if ($mode == COMMENT_MODE_FLAT_COLLAPSED || $mode == COMMENT_MODE_FLAT_EXPANDED) {
      $query .= ' ORDER BY c.cid DESC';
    }
    else {
      $query .= ' ORDER BY c.thread DESC';
    }
  }
  else if ($order == COMMENT_ORDER_OLDEST_FIRST) {
    if ($mode == COMMENT_MODE_FLAT_COLLAPSED || $mode == COMMENT_MODE_FLAT_EXPANDED) {
      $query .= ' ORDER BY c.cid';
    }
    else {
      // See comment above. Analysis reveals that this doesn't cost too
      // much. It scales much much better than having the whole comment
      // structure.
      $query .= ' ORDER BY SUBSTRING(c.thread, 1, (LENGTH(c.thread) - 1))';
    }
  }
  $query = db_rewrite_sql($query, 'c', 'cid');
  $query_count = db_rewrite_sql($query_count, 'c', 'cid');
  $num_of_comments = db_result(db_query($query_count, $nid));

  if (($num_of_comments < $comments_per_page) || ($comments_per_page * ($page) > $num_of_comments)) {
    return;
  }

  // Start a form, for use with comment control.
  if (isset($page)) {
    $_GET['page'] = $page;
  }
  $result = pager_query($query, $comments_per_page, 0, $query_count, $query_args);

  $divs = 0;
  $num_rows = FALSE;
  $comments = '';

  while ($comment = db_fetch_object($result)) {
    $comment = drupal_unpack($comment);
    $comment->name = $comment->uid ? $comment->registered_name : $comment->name;
    $comment->depth = count(explode('.', $comment->thread)) - 1;

    if ($mode == COMMENT_MODE_THREADED_COLLAPSED || $mode == COMMENT_MODE_THREADED_EXPANDED) {
      if ($comment->depth > $divs) {
        $divs++;
        $comments .= '<div class="indented">';
      }
      else {
        while ($comment->depth < $divs) {
          $divs--;
          $comments .= '</div>';
        }
      }
    }

    if ($mode == COMMENT_MODE_FLAT_COLLAPSED) {
      $comments .= theme('comment_flat_collapsed', $comment, $node);
    }
    elseif ($mode == COMMENT_MODE_FLAT_EXPANDED) {
      $comments .= theme('comment_flat_expanded', $comment, $node);
    }
    elseif ($mode == COMMENT_MODE_THREADED_COLLAPSED) {
      $comments .= theme('comment_thread_collapsed', $comment, $node);
    }
    elseif ($mode == COMMENT_MODE_THREADED_EXPANDED) {
      $comments .= theme('comment_thread_expanded', $comment, $node);
    }

    $num_rows = TRUE;
  }
  while ($divs-- > 0) {
    $comments .= '</div>';
  }

  $output .= $comments;
  $next = $page + 1;

  if ($comments_per_page * $next < $num_of_comments) {
    $output .= theme('morecomments_pager', $nid, $next, $node->type);
  }

  print $output;
  exit();
}
